# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vietnam_number',
 'vietnam_number.number2word',
 'vietnam_number.number2word.utils',
 'vietnam_number.word2number',
 'vietnam_number.word2number.utils']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'vietnam-number',
    'version': '1.0.3',
    'description': 'Thư viện xữ lý chữ số dành riêng cho Tiếng Việt - Hỗ trợ chuyển đổi cho cả văn nói và văn viết',
    'long_description': "====================================\nVietnam Number Toolkit\n====================================\n\n.. image:: https://madewithlove.now.sh/vn?heart=true&colorB=%23d5481d\n        :target: https://pypi.python.org/pypi/vietnam-number\n\n.. image:: https://img.shields.io/pypi/v/vietnam-number\n        :target: https://pypi.python.org/pypi/vietnam-number\n\n.. image:: https://img.shields.io/badge/python-3.6%20%7C%203.7%20%7C%203.8-blue\n        :target: https://pypi.python.org/pypi/vietnam-number\n\n.. image:: https://img.shields.io/badge/license-GPLv3-brightgreen.svg\n        :target: https://pypi.python.org/pypi/vietnam-number\n\n.. image:: https://img.shields.io/badge/chat-on%20facebook-informational\n        :target: https://www.facebook.com/ng.thuong45/\n\n|\n\n.. image:: https://user-images.githubusercontent.com/66929261/100044430-424d1800-2e42-11eb-8875-3b5abbf25ade.png\n\n**Vietnam-number** là một thư viện Python mã nguồn mở hỗ trợ cho việc xữ lý chữ số trong Tiếng Việt.\n\n+---------------------+------------------------------------------------------------------------------------------------+\n|    **Support**      |                           **Description**                                                      |\n+---------------------+------------------------------------------------------------------------------------------------+\n| Văn nói - Văn viết  | * Hỗ trợ cho cả văn viết và văn nói thông dụng.                                                |\n+---------------------+------------------------------------------------------------------------------------------------+\n| Số có giá trị lớn   | * Hỗ trợ chuyển đổi chử số từ 0 đến 999.999.999.999                                            |\n+---------------------+------------------------------------------------------------------------------------------------+\n| Từ ngữ dân gian     | * Hỗ trợ các từ ngữ dân gian, vùng miền:                                                       |\n|                     | * Vd: 'tỷ - tỏi', 'triệu - chai, củ', 'nghìn - ngàn', 'trăm - lít' , 'bốn - tư'...             |\n+---------------------+------------------------------------------------------------------------------------------------+\n| Cách đọc đơn        | * Hỗ trợ chuyển đổi văn bản chữ số sang số theo cách đọc từng số:                              |\n|                     | * Vd: 'không một hai ba bốn năm sáu bảy tám chín' = 0123456789                                 |\n+---------------------+------------------------------------------------------------------------------------------------+\n| Cách đọc đôi        | * Hỗ trợ chuyển đổi văn bản chữ số sang số theo cách đọc từng cặp số:                          |\n|                     | * Vd: 'không một hai ba bốn mươi lăm sáu mươi bảy tám mươi chín' = 0123456789                  |\n+---------------------+------------------------------------------------------------------------------------------------+\n\nCài đặt\n----------------------------------------\n\nCài đặt vietnam-number bằng lệnh:\n\n| **$ pip install vietnam-number**\n\nTính năng\n----------------------------------------\n\n| `1. Chữ số sang số có từ liên kết.`_\n| `2. Chữ số sang số không có từ liên kết.`_\n| `3. Số sang chữ số có từ liên kết`_\n| `4. Số sang chữ số không có từ liên kết`_\n\n****************************************\n1. Chữ số sang số có từ liên kết.\n****************************************\n\n.. image:: https://img.shields.io/badge/feature-word%20to%20number-orange\n\n| Chuyển đổi **Chữ số** sang **Số** có các từ liên kết ('mươi', 'trăm', 'nghìn', 'triệu', 'tỷ').\n| Sử dụng phù hợp trong các tình huống như: đơn vị tiền tệ, số tuổi...\n\n.. code-block:: python\n\n    >>> # -*- coding: utf-8 -*-\n    >>> from vietnam_number import w2n\n    >>> text = 'một triệu không trăm tám mươi lăm nghìn ba trăm ba mươi hai'\n\n    >>> w2n(text)\n    1085332\n\n    # Văn nói vắn tắt\n    >>> text_1 = 'một triệu tư'\n    >>> text_2 = 'nghàn hai'\n\n    >>> w2n(text_1)\n    1400000\n\n    >>> w2n(text_2)\n    1200\n\n    # Hỗ trợ số có giá trị lớn\n    >>> text = 'chín trăm năm mươi ba tỷ lẽ tám mươi bảy triệu'\n\n    >>> w2n(text)\n    953087000000\n\n    # Văn nói đặt biệt\n    # Chỉ áp dụng cho chữ số có từ liên kết.\n    # Trường hợp không có từ liên kết chỉ áp dụng cho các số có giá trị từ 0 đến 999\n    >>> text_1 = 'ba bốn mươi hai'\n    >>> text_2 = 'ba mươi bảy năm'\n\n    >>> w2n(text_1)\n    342\n\n    >>> w2n(text_1)\n    375\n\n    # Từ ngữ dân gian\n    >>> text = 'bảy củ'\n\n    >>> w2n(text)\n    7000000\n\n****************************************\n2. Chữ số sang số không có từ liên kết.\n****************************************\n\n.. image:: https://img.shields.io/badge/feature-word%20to%20number-orange\n\n| Chuyển đổi **Chữ số** sang **Số** không có từ liên kết.\n| Sử dụng phù hợp trong các tình huống một dãy nhiều chữ số khác nhau như: số chứng minh thư, số visa, số thẻ ATM, số điện thoại, tài khoản ngân hàng...\n\n.. code-block:: python\n\n    >>> # -*- coding: utf-8 -*-\n    >>> from vietnam_number import w2n_single, w2n_couple\n\n    # Văn nói cho cách đọc đơn từng số một.\n    >>> text = 'không tám không chín một hai ba bốn năm sáu'\n\n    >>> w2n_single(text)\n    0809123456\n\n    # Cách đọc đôi từng cặp số.\n    # Bắt buộc số ban đầu phải là số kết hợp bởi từng cặp số,\n    # hay nói cách khác số ban đầu phải có số lượng chữ số chia hết cho 2.\n    # Trong trường hợp đặt biệt này, dãy số có thể có hoặc không có từ liên kết hàng chục là từ 'mươi'...\n    # vd:\n    #  032 -> 'không ba mươi hai' -> không được phép (số lượng phần tử số ban đầu lẽ) -> đầu ra không chính xác.\n    #  0324 -> 'không ba mươi hai bốn' -> không được phép (không phải cách đọc từng cặp số một) -> đầu ra không chính xác\n    #  0324 -> 'không ba hai mươi bốn' -> Ok\n    >>> text = 'hai mươi ba bảy tám mươi bốn năm bốn chín mươi mốt mười hai bảy năm'\n\n    >>> w2n_couple(text)\n    20378454911275\n\n\n****************************************\n3. Số sang chữ số có từ liên kết\n****************************************\n\n.. image:: https://img.shields.io/badge/feature-number%20to%20word-yellow\n\n| Chuyển đổi **Số** sang **Chữ số** cần có các từ liên kết như ('mươi', 'trăm', 'nghìn', 'triệu', 'tỷ').\n| Sử dụng phù hợp trong các trường hợp như: giá sản phẩm, đơn vị tiền tệ, số tuổi...,\n| Hỗ trợ văn nói cho các từ như 'một - mốt', 'năm - lăm', 'lẽ - linh'...\n\n.. code-block:: python\n\n    >>> # -*- coding: utf-8 -*-\n    >>> from vietnam_number import n2w\n\n    >>> number = '115205201211'\n    >>> n2w(number)\n    'một trăm mười lăm tỷ hai trăm lẽ năm triệu hai trăm lẽ một nghìn hai trăm mười một'\n\n\n    # hỗ trợ một số trường hợp có cách đọc đặc biệt\n    >>> list_number = ['111', '200', '101', '121', '815', '805', '825']\n    >>> for element in list_number:\n    ...      print(n2w(element))\n    ...\n    'một trăm mười một'\n    'hai trăm'\n    'một trăm lẽ một'\n    'một trăm hai mươi mốt'\n    'tám trăm mười lăm'\n    'tám trăm lẽ năm'\n    'tám trăm hai mươi lăm'\n\n\n****************************************\n4. Số sang chữ số không có từ liên kết\n****************************************\n\n.. image:: https://img.shields.io/badge/feature-number%20to%20word-yellow\n\n| Chuyển đổi **Số** sang **Chữ số** không cần có các từ liên kết.\n| Sử dụng phù hợp trong các trường hợp một chuỗi nhiều số khác nhau như: số chứng minh thư, số visa, số thẻ ATM, số điện thoại, tài khoản ngân hàng...\n\n.. code-block:: python\n\n    >>> # -*- coding: utf-8 -*-\n    >>> from vietnam_number import n2w_single\n\n    # Chuyển đổi từng số một.\n    >>> number = '0908123456'\n    >>> n2w_single(number)\n    'không chín không tám một hai ba bốn năm sáu'\n\n    # Hỗ trợ cho số có đầu số là '+84'\n    >>> number = '+84908123456'\n    >>> n2w_single(number)\n    'không chín không tám một hai ba bốn năm sáu'\n\n\nTính Năng Trong Tương Lai\n----------------------------------------\n\n* Hỗ trợ số thập phân.\n* Hỗ trợ số âm.\n* Chuyển đổi số thứ tự\n* Chuyển đổi ngày tháng năm\n* Chuyển đổi đơn vị đo khối lượng (g - kg - yến - tạ - tấn)\n* Chuyển đổi đơn vị đo độ dài (mm - cm - m - km)\n\nĐóng Góp\n----------------------------------------\n\nThông báo lỗi :beetle: tại https://github.com/ngthuong45/vietnam-number/issues\n\nNếu bạn phát hiện một lỗi mới, vui lòng thông báo đính kèm các thông tin:\n\n* Thông tin hệ điều hành của bạn.\n* Những thiết lập ở local nếu có.\n* Các bước chi tiết dẫn đến phát hiện lỗi.\n\n|\n\n*Để lại một sao :star: nếu thư viện giúp ích được phần nào cho công việc của bạn nhé!.*\n",
    'author': 'Nguyễn Hoài Thương',
    'author_email': 'ng.thuong.lio@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ngthuong45/vietnam-number.git',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
